###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen <morten@thingamy.net>
#	Copyright (c) 2000,2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

__author__ = '$Author: morphex $'
__version__ = '$Revision: 1.6 $'


import imaplib
import string
import cStringIO
import mail_wrapper


class imap4_wrapper(mail_wrapper.mail_wrapper):
	"""A protocol wrapper."""

	def __init__(self,server,port=None):

		mail_wrapper.mail_wrapper.__init__(self,server,port)
        
		if self.port:
			self.connection = imaplib.IMAP4(self.server,self.port)
		else:
			self.connection = imaplib.IMAP4(self.server)

	def login(self, username, password):
        	"""Logins to IMAP server."""

		self.connection.login(username,password)
		self.connection.select()

	def list(self):
		"""Returns a list of unique message ids."""

		result = self.connection.uid('SEARCH','ALL')
		return map(int, string.split(result[-1][-1]))
            
	def delete(self, unique_message_id):
		"""Marks message as deleted."""

		self.connection.uid('STORE',
			str(unique_message_id),'+FLAGS','\\DELETED')

	def retrieve(self, unique_message_id):
		"""Retrieves specified message as a dummy mailbox message."""

		message = self.connection.uid('FETCH',
			str(unique_message_id),'(RFC822)')[-1][0][1]
		return mail_wrapper.dummy_mailbox_message(
			cStringIO.StringIO(message))

	def head(self, unique_message_id):
		"""Retrieves message header."""

		return self.connection.uid('FETCH',
			str(unique_message_id),'HEAD')

	def quit(self, expunge=1):
		"""Quits the connection.  If the expunge parameter is true,
		all messages marked as deleted are deleted."""

		if expunge:
			self.connection.expunge()

		self.connection.logout()


class imap4_session(imap4_wrapper):

	def __init__(self, server, port=None, delete_retrieved_messages=1,
			capture_mailbox_structure=None):

		imap4_wrapper.__init__(self, server, port)
		self.delete_retrieved_messages = delete_retrieved_messages
		self.capture_mailbox_structure = capture_mailbox_structure
		self.retrieved_messages = []

	def login(self, username, password):
		"""Logs in the user and updates the message id list."""

		# This function doesn't really do it for us, as some
		# mailboxes may contain the same number of messages and
		# don't provide account-wide unique message ids.

		def sequence_is_equal(first_sequence, second_sequence):
			"""Quick little function that will compare the
			contents of two sequences."""

			for item_index in range(len(first_sequence)):
				try:
					if not first_sequence[item_index] == \
						second_sequence[item_index]:
						break
				except IndexError:
					break

			else:
				return 1

			return 0

		# We have to override the login method here so that
		# we can update the id list and possibly make a
		# mailbox -> message_id structure.

		imap4_wrapper.login(self, username, password)

		if self.capture_mailbox_structure:

			self.mailboxes = []
			items = self.connection.list()[-1]

			for item in items:
				# We do this so we can skip around
				# quoted mailbox names
				if item[-1] == '"':
					item = item[string.rfind(item,
						'"', 0, len(item) - 1)+1:-1]
				else:
					# This may not work when we're
					# handling mailbox names with spaces
					# in them.  Look at the possibility
					# of searching for the last quote 
					# (often "/") and adding a bit of
					# extra logic
					item = item[string.rfind(item,
						' ', 0, len(item) - 1)+1:]
				try:				
					self.connection.select(item)

					# Some IMAP servers list the INBOX
					# mailbox twice.. We still need to
					# add logic that checks whether the
					# given mailbox has been captured,
					# as INBOX can be an alias for
					# another mailbox

					if not item in map(lambda x: x[0],
						self.mailboxes):
						self.mailboxes.append(item,
							string.split(
							self.connection.\
							search(None,
								'ALL')[-1][0]))

				except self.connection.readonly:
					pass

		else:
			self.connection.select()
			self.id_list = self.list()


	def next(self):
		"""Returns the next message in the list."""

		if not self.capture_mailbox_structure:

			try:
				message_id = self.id_list.pop()
			except IndexError:
				return None

			message = self.retrieve(message_id)
			self.retrieve_messages.append(message_id)
			return message

		else:

			try:
				mailbox = self.mailboxes[-1]
			except IndexError:
				return None

			try:
				self.connection.select(mailbox[0])
				message_id = mailbox[-1].pop()
				message = self.retrieve(message_id)
				self.retrieved_messages.append((mailbox[0],
					message_id))
				return (string.split(mailbox[0], '/'), message)

			except IndexError:

				self.mailboxes.pop()
				# If there aren't any mailboxes on the next
				# try, we'll return None
				return self.next()


	def quit(self):
		"""Deleted retrieved messages if specified and closes
		the connection."""

		if self.delete_retrieved_messages:

			if self.capture_mailbox_structure:

				mailbox_ = None

				for mailbox, message_id in \
					self.retrieved_messages:
					if mailbox != mailbox_:
						self.connection.select(mailbox)
						self.delete(message_id)

			else:
				for message_id in self.retrieved_messages:
					self.delete(message_id)

			self.connection.expunge()

		self.connection.logout()


if __name__ == '__main__':

	session = imap4_session('localhost', capture_mailbox_structure=1,
				delete_retrieved_messages=0)
	session.login('test123','test123')

	messages = []

	while 1:

		message = session.next()

		if message is not None:

			messages.append(message)

		else:

			break

	#wrapper = imap4_wrapper('localhost')
	#wrapper.login('test123','test123')

