###########################################################################
#
#   Copyright (c) 2001 Morten W. Petersen <morten@thingamy.net>
#	Copyright (c) 2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morphex $
# $Revision: 1.5 $

from utils import html_quote
html_quote_function = html_quote
import types

class html_widget_base:
    """Base class for HTML widgets."""

    type = 'html_widget_base'

    def render(self, *args, **kw):
        """Returns an HTML presentation of the widget."""
        return apply(self.__call__, args, kw)

    def is_type(self, type):
        """Returns a truth value if the widget is of the given type."""
        return self.type == type


class checkbox(html_widget_base):
    """Class for handling <input type="checkbox"> tags."""

    type = 'checkbox'

    def __init__(self, name='', value='', displayed_value='', checked=None):
        self.name = name
        self.value = value
        self.displayed_value = displayed_value or value
        self.checked = checked

    def __call__(self, html_quote=1):
        """Returns an HTML presentation of the widget."""
        if html_quote:
            value = html_quote_function(self.value)
            displayed_value = html_quote_function(self.displayed_value)
        else:
            value = self.value
            displayed_value = self.displayed_value
        if self.checked:
            checked = " checked"
        else:
            checked = ""
        return """<input type="checkbox" name="%s" value="%s"%s> %s""" % \
                (self.name, value, checked, displayed_value)

class password(html_widget_base):
    """Class for handling <input type="password"> tags."""

    type = 'password'

    def __init__(self, name='', *args, **kw):
        self.name = name

class hidden(html_widget_base):
    """Class for handling <input type="hidden"> tags."""

    type = 'hidden'

    def __init__(self, name='', value='', *args, **kw):
        self.name = name
        self.value = value

    def __call__(self, html_quote=1):
        """Returns an HTML presentation of the widget."""
        if html_quote:
            value = html_quote_function(self.value)
        else:
            value = self.value
        return """<input type="hidden" name="%s" value="%s" />""" % \
            (self.name, value)

class image(html_widget_base):
    """Class for handling <input type="image"> tags."""

    type = 'image'

    def __init__(self, name='', *args, **kw):
        self.name = name

class reset(html_widget_base):
    """Class for handling <input type="reset"> tags."""

    type = 'reset'

    def __init__(self, name='', *args, **kw):
        self.name = name

class submit(html_widget_base):
    """Class for handling <input type="submit"> tags."""

    type = 'submit'

    def __init__(self, name='', *args, **kw):
        self.name = name

class file(html_widget_base):
    """Class for handling <input type="file"> tags."""

    type = 'file'

    def __init__(self, name='', *args, **kw):
        self.name = name

class button(html_widget_base):
    """Class for handling <input type="button"> tags."""

    type = 'button'

    def __init__(self, name='', value='', tabindex='',
            accesskey='', onfocus='', onblur=''):
        self.name = ''

class text(html_widget_base):
    """Class for handling <input type="text"> tags."""

    type = 'text'

    def __init__(self, name=None, value=None, size=None,
            onclick=None, onchange=None):
        """Initalizies the text widget."""
        self.name = name
        self.value = value
        self.size = size or 40
        self.onclick = onclick or None
        self.onchange = onchange or None

    def __call__(self, html_quote=1):
        """Returns an HTML presentation of the widget."""
        if html_quote:
            value = html_quote_function(self.value)
        else:
            value = self.value
        return """<input type="text" name="%s" size="%s" value="%s" />""" % \
                (self.name, self.size, self.value)

class textarea(html_widget_base):
    """Class for handling <textarea> tags."""

    type = 'textarea'

    def __init__(self, name='', value='', rows=None, cols=None):
        """Initializes the textarea widget."""
        self.name = name
        self.value = value
        self.rows = rows or 10
        self.cols = cols or 50


    def __call__(self, html_quote=1):
        """Returns an HTML presentation of the widget."""
        if html_quote:
            value = html_quote_function(self.value)
        else:
            value = self.value
        return """<textarea name="%s" rows="%s" cols="%s">%s</textarea>""" % \
                (self.name, self.rows, self.cols, value)

class select_option(html_widget_base):
    """Class used for handling <option> tags within <select> tags."""

    type = 'select_option'

    def __init__(self, value, displayed_value=None,
            selected=None):
        """Initializes the option."""
        self.value = value
        if displayed_value is None:
            self.displayed_value = self.value
        else:
            self.displayed_value = displayed_value
        self.selected = selected

    def select(self):
        """Sets the selected flag to true."""
        self.selected = 1

    def unselect(self):
        """Sets the selected flag to false."""
        self.selected = 0

    def __call__(self, html_quote=1):
        """Returns an HTML presentation."""
        if self.selected:
            selected = " selected"
        else:
            selected = ""
        if html_quote:
            value = html_quote_function(self.value)
            displayed_value = html_quote_function(self.displayed_value)
        else:
            value = self.value
            displayed_value = self.displayed_value
        return """<option value="%s"%s>%s</option>""" % \
                (value, selected, displayed_value)

class radio(html_widget_base):
    """Class for handling a single <input type="radio"> within a group of them."""

    type = 'radio'

    def __init__(self, name='', value='', displayed_value=None,
            checked=None):
        """Initializes the option."""
        self.name = name
        self.value = value
        if displayed_value is None:
            self.displayed_value = value
        else:
            self.displayed_value = displayed_value
        self.checked = checked

    def check(self):
        """Sets the checked flag to true."""
        self.checked = 1

    def uncheck(self):
        """Sets the checked flag to false."""
        self.checked = 0

    def __call__(self, html_quote=1):
        """Returns an HTML presentation."""
        if self.checked:
            checked = " checked"
        else:
            checked = ""
        if html_quote:
            value = html_quote_function(self.value)
            displayed_value = html_quote_function(self.displayed_value)
        else:
            value = self.value
            displayed_value = displayed_value
        return """<input type="radio" name="%s" value="%s"%s />%s""" % \
                (self.name, self.value, checked, self.displayed_value)

class radio_collection(html_widget_base):
    """Class used for handling a selection of <input type="radio"> tags."""

    type = 'radio_collection'

    def __init__(self, name='', true='', false=''):
        """Initialize the widget."""
        self.name = name
        self.true = true or 'Yes'
        self.false = false or 'No'
        self.options = []

    def add_option(self, value='', displayed_value='', checked=None):
        self.options.append(radio(name=self.name, value=value, displayed_value=displayed_value,
                        checked=checked))

    def __call__(self, html_quote=1):
        """Returns an HTML presentation of the widget."""
        html = ""
        for option in self.options:
            html = html + option.render(html_quote=html_quote)
        return html

class checkbox_collection(html_widget_base):
    """Class used for handling sets of <input type="checkbox"> tags."""

    type = 'checkbox_collection'

    def __init__(self, name=''):
        """Initializes the widget."""
        self.name = name
        self.options = []

    def add_option(self, value='', displayed_value='', checked=None):
        """Adds an option to the collection."""
        self.options.append(checkbox(name=self.name, value=value,
                displayed_value=displayed_value, checked=checked))

    def __call__(self, html_quote=1, separator="<br>", use_columns=1, column_width=4, column_trigger=20):
        """Returns an HTML presentation of the widget."""
        if len(self.options) > column_trigger and use_columns:
            html = '<table border="0" width="100%"><tr>'
            use_columns = 1
        else:
            html = ''
            use_columns = 0
        for option_index in range(len(self.options)):
            option = self.options[option_index]
            if use_columns:
                html = html + '<td>' + option(html_quote=html_quote) + '</td>'
                if not (option_index + 1) % column_width:
                    html = html + '</tr><tr>'
            else:
                html = html + option(html_quote=html_quote) + separator
        else:
            if use_columns:
                html = html + '</tr></table>'
        return html

class select(html_widget_base):
    """Class for handling <select> tags."""

    type = 'select'

    def __init__(self, name='', multiple=None, size=1):
        """Initializes the widget."""
        self.name = name
        self.options = []
        self.multiple = multiple
        self.size = size

    def add_option(self, option):
        """Adds an option to the selection."""
        self.options.append(option)

    def __call__(self, html_quote=1):
        """Returns an HTML presentation of the widget."""
        if self.multiple:
            multiple = " multiple"
            # 'Clever', isn't it?  :-\
            if not self.size:
                self.size = 3
            elif self.size < 2:
                self.size = 3
        else:
            multiple = ""
        if self.size:
            size = ' size="%s"' % self.size
        else:
            size = ''
        html = """<select name="%s"%s%s>""" % (self.name, multiple, size)
        for option in self.options:
            html = html + option(html_quote=html_quote)
        return html + "</select>"

def make_select(name='', options=[], multiple=None, size=1, selected_values=[]):
    widget = select(name=name, multiple=multiple, size=size)
    if not type(options[0]) in (types.ListType, types.TupleType):
        # If we've got a simple sequence
        options = map(lambda x: (x, x), options)
    for (option_key, option_value) in options:
        option = select_option(option_key, option_value)
        if multiple:
            if option_key in selected_values:
                option.select()
        else:
            if option_key == selected_values:
                option.select()
        widget.add_option(option)
    return widget

def make_checkbox_collection(name='', options=[], selected_values=[]):
    widget = checkbox_collection(name=name)
    if not type(options[0]) in (types.ListType, types.TupleType):
        # If we've got a simple sequence
        options = map(lambda x: (x, x), options)
    for (option_key, option_value) in options:
        if option_key in selected_values:
            checked = 1
        else:
            checked = 0
        widget.add_option(
            value=option_key,
            displayed_value=option_value,
            checked=checked)
    return widget